const path = require('path');
const slug = require('slug');
const unslug = require('unslug');
const { exec } = require('child-process-promise');

module.exports = [
  {
    name: 'projectName',
    placeholder: '%%{CREATE_IVX_APP_PROJECT_NAME}%%',
    type: 'input',
    message: 'Project Title:',
    default: () => unslug(path.basename(process.cwd())),
  },
  {
    name: 'projectNameSlug',
    placeholder: '%%{CREATE_IVX_APP_PROJECT_NAME_SLUG}%%',
    type: 'input',
    message: 'Project Name:',
    validate: input =>
      /^([A-Za-z\-_\d])+$/.test(input)
        ? true
        : 'Project name may only include letters, numbers, underscores and hashes.',
    default: ({ projectName }) => slug(projectName).toLowerCase(),
  },
  {
    name: 'projectDescription',
    placeholder: '%%{CREATE_IVX_APP_PROJECT_DESCRIPTION}%%',
    type: 'input',
    message: 'Project Description:',
  },
  {
    name: 'projectAuthor',
    placeholder: '%%{CREATE_IVX_APP_PROJECT_AUTHOR}%%',
    type: 'input',
    message: 'Author:',
    validate: input =>
      /([^<]+)?(?:<?(.+@[^>]+)>?)/.test(input)
        ? true
        : 'Author must be set in the format "John Doe<john.doe@example.com>" (without ").',
    default: () =>
      new Promise(async (resolve, reject) => {
        const { stdout: name } = await exec('git config user.name').catch(
          reject,
        );
        const { stdout: email } = await exec('git config user.email').catch(
          reject,
        );
        resolve(`${name.trim()}<${email.trim()}>`);
      }),
  },
  {
    name: 'projectPort',
    placeholder: '%%{CREATE_IVX_APP_PROJECT_PORT}%%',
    type: 'input',
    message: 'Local Port:',
    validate: input =>
      /^([0-9])+$/.test(input) ? true : 'Port has to be a number.',
    default: 9001,
  },
];
