#!/usr/bin/env node
const fs = require('fs-extra');
const readDir = require('recursive-readdir');
const placeholders = require('./placeholders');

// Set create-ivx-app placeholders
const defaultValues = {
  '%%{CREATE_IVX_APP_PROJECT_NAME}%%': 'my-project',
  '%%{CREATE_IVX_APP_PROJECT_NAME_SLUG}%%': 'my-project-name',
  '%%{CREATE_IVX_APP_PROJECT_DESCRIPTION}%%': 'My Project description',
  '%%{CREATE_IVX_APP_PROJECT_AUTHOR}%%': 'Me <me@me.com>',
  '%%{CREATE_IVX_APP_PROJECT_PORT}%%': 9002,
};

const replacePlaceholdersInFile = (filePath, values = defaultValues) =>
  new Promise((resolve, reject) => {
    fs.readFile(filePath, (err, file) => {
      if (err) {
        console.error('Error while trying to read file: ', err);
        reject(err);
      }
      let content = file.toString();
      Object.entries(placeholders).forEach(([key, placeholder]) => {
        content = content.split(placeholder).join(values[key]);
      });
      fs.writeFile(filePath, content, writeError => {
        if (writeError) {
          console.error('Error while trying to write file: ', writeError);
          reject(writeError);
        }
        resolve();
      });
    });
  });

const asyncForEach = async (array, callback) => {
  for (let index = 0; index < array.length; index += 1) {
    // eslint-disable-next-line no-await-in-loop
    await callback(array[index], index, array);
  }
};

module.exports = settings =>
  new Promise(async (resolve, reject) => {
    const allFiles = await readDir(settings.projectNameSlug);
    await asyncForEach(
      allFiles,
      async file =>
        // eslint-disable-next-line no-return-await
        await replacePlaceholdersInFile(file, settings).catch(reject),
    );
    resolve();
  });
