const {
  printWithType,
  printReceived,
  printExpected,
  matcherHint,
  RECEIVED_COLOR,
} = require('jest-matcher-utils');
const fs = require('fs-extra');

const ensureMock = (mockOrSpy, matcherName) => {
  if (
    !mockOrSpy ||
    ((mockOrSpy.calls === undefined || mockOrSpy.calls.all === undefined) &&
      // eslint-disable-next-line no-underscore-dangle
      mockOrSpy._isMockFunction !== true)
  ) {
    throw new Error(
      `${matcherHint(`[.not]${matcherName}`, 'jest.fn()', '')}\n\n` +
        `${RECEIVED_COLOR('jest.fn()')} value must be a mock function ` +
        `or spy.\n${printWithType('Received', mockOrSpy, printReceived)}`,
    );
  }
};

const isSpy = spy => spy.calls && typeof spy.calls.count === 'function';

expect.extend({
  toBeExistingPath(receivedPath) {
    const pass = fs.pathExistsSync(receivedPath);
    return {
      message: () =>
        `expected ${printReceived(receivedPath)}${
          pass ? ' not' : ''
        } to be an existing path`,
      pass,
    };
  },
});

expect.extend({
  toBeCalledWithArgumentContaining(received, ...expected) {
    ensureMock(received, 'toHaveBeenCalledWithArgumentContaining');

    const receivedIsSpy = isSpy(received);
    const type = receivedIsSpy ? 'spy' : 'mock function';
    const receivedName = receivedIsSpy ? 'spy' : received.getMockName();

    const identifier =
      receivedIsSpy || receivedName === 'jest.fn()'
        ? type
        : `${type} "${receivedName}"`;

    const calls = receivedIsSpy
      ? received.calls.all().map(x => x.args)
      : received.mock.calls;

    let pass = false;
    calls.forEach(call => {
      call.forEach(arg => {
        expected.forEach(expectedValue => {
          if (typeof arg === 'string' && arg.indexOf(expectedValue) > 0)
            pass = true;
        });
      });
    });

    const message = pass
      ? () =>
          `${matcherHint(
            '.not.toHaveBeenCalledWithArgumentContaining',
            receivedName,
          )}\n\n` +
          `Expected ${identifier} not to have been called with argument containing:\n` +
          `  ${printExpected(expected)}`
      : () =>
          `${matcherHint(
            'toHaveBeenCalledWithArgumentContaining',
            receivedName,
          )}\n\n` +
          `Expected ${identifier} to have been called with argument containing:\n${printExpected(
            expected,
          )}`;

    return { message, pass };
  },
});
