const path = require('path');
const fs = require('fs-extra');
const readDir = require('recursive-readdir');
const copyTemplate = require('../src/copy-template');
const mockAnswers = require('./fixtures/answers');

describe('copy-templates', () => {
  afterEach(async () => {
    await fs.remove(mockAnswers.projectNameSlug).catch(err => {
      throw err;
    });
    expect(mockAnswers.projectNameSlug).not.toBeExistingPath();
  });

  afterAll(async () => {
    await fs.remove(mockAnswers.projectNameSlug).catch(err => {
      throw err;
    });
    expect(mockAnswers.projectNameSlug).not.toBeExistingPath();
  });

  it('creates folder specified in passed argument', async () => {
    const copySpy = jest.spyOn(fs, 'copy');
    await copyTemplate(mockAnswers);
    expect(copySpy).toHaveBeenCalledWith(
      path.join(process.cwd(), 'templates', mockAnswers.template),
      path.join(process.cwd(), mockAnswers.projectNameSlug),
    );

    expect(mockAnswers.projectNameSlug).toBeExistingPath();
  });

  it('copies the content from template into folder', async () => {
    await copyTemplate(mockAnswers);
    const templatePath = path.join('templates', mockAnswers.template);

    const allFiles = await readDir(templatePath);
    const filePaths = allFiles.map(file => file.replace(templatePath, ''));

    filePaths.forEach(file =>
      expect(path.join(mockAnswers.projectNameSlug, file)).toBeExistingPath(),
    );
  });
});
