const path = require('path');
const fs = require('fs-extra');
const mockAnswers = require('./fixtures/answers');
const installDependencies = require('../src/install-dependencies');

const examplePackageJsonPath = path.join(
  'tests',
  'fixtures',
  'example-package.json',
);
const tempFolder = 'temp-test-folder';

describe('install-dependencies', () => {
  beforeEach(async () => {
    // copy fixture folder to temp folder to test modifications on them
    await fs.ensureDir(tempFolder).catch(err => {
      throw err;
    });
    expect(tempFolder).toBeExistingPath();

    await fs
      .copy(examplePackageJsonPath, path.join(tempFolder, 'package.json'))
      .catch(err => {
        throw err;
      });
    expect(path.join(tempFolder, 'package.json')).toBeExistingPath();
  });

  afterEach(async () => {
    // remove temp folder
    await fs.remove(tempFolder).catch(err => {
      throw err;
    });
    expect(tempFolder).not.toBeExistingPath();
  });

  afterAll(async () => {
    // This is to make sure we do teardown even after a test fails
    await fs.remove(tempFolder).catch(err => {
      throw err;
    });
    expect(tempFolder).not.toBeExistingPath();
  });

  it('installs dependencies from a example-package.json file', async () => {
    jest.setTimeout(60000); // because who knows how long npm may take
    await installDependencies({ ...mockAnswers, projectNameSlug: tempFolder });

    expect(
      path.join(tempFolder, 'node_modules', 'classnames'),
    ).toBeExistingPath();
    expect(path.join(tempFolder, 'node_modules', 'idx')).toBeExistingPath();
  });
});
