const path = require('path');
const fs = require('fs-extra');
const readDir = require('recursive-readdir');
const mockAnswers = require('./fixtures/answers');
const placeholders = require('../src/placeholders');
const replacePlaceholders = require('../src/replace-placeholders');

const fixturePath = path.join('tests', 'fixtures', 'template');
const tempFolder = 'temp-test-folder';

describe('replace-placeholders', () => {
  beforeEach(async () => {
    // copy fixture folder to temp folder to test modifications on them
    await fs.copy(fixturePath, tempFolder).catch(err => {
      throw err;
    });
    expect(tempFolder).toBeExistingPath();

    const allFiles = await readDir(fixturePath);
    const filePaths = allFiles.map(file => file.replace(fixturePath, ''));

    filePaths.forEach(file =>
      expect(path.join(tempFolder, file)).toBeExistingPath(),
    );
  });

  afterEach(async () => {
    // remove temp folder
    await fs.remove(tempFolder).catch(err => {
      throw err;
    });
    expect(tempFolder).not.toBeExistingPath();
  });

  afterAll(async () => {
    // This is to make sure we do teardown even after a test fails
    await fs.remove(tempFolder).catch(err => {
      throw err;
    });
    expect(tempFolder).not.toBeExistingPath();
  });

  it('replaces placeholders in specific file', async () => {
    await replacePlaceholders({ ...mockAnswers, projectNameSlug: tempFolder });

    const allFiles = await readDir(tempFolder);

    allFiles.forEach(filePath => {
      fs.readFile(filePath, (err, file) => {
        if (err) {
          console.error('Error while trying to read file: ', err);
          throw err;
        }
        const content = file.toString();
        Object.values(placeholders).forEach(placeholder => {
          expect(content).not.toContain(placeholder);
        });
      });
    });
  });
});
